@@ -1,12 +1,12 @@
 /**********************************************************************
  * vanessa_socket.h                                       November 1999
- * Horms                                             horms@verge.net.au
+ * Simon Horman                                      horms@verge.net.au
  *
  * Open a tcp socket to a server (we are the client)
  *
  * vanessa_socket
  * Library to simplify handling of TCP sockets
- * Copyright (C) 1999-2004  Horms
+ * Copyright (C) 1999-2008  Simon Horman <horms@verge.net.au>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -47,6 +47,10 @@
 
 #include <vanessa_logger.h>
 
+#ifndef SOMAXCONN
+#define SOMAXCONN 5
+#endif
+
 /* Needed for Solaris */
 #ifdef sun
 #define timeradd(a, b, result)                                                \
@@ -77,6 +81,7 @@ typedef unsigned int vanessa_socket_flag_t;
 #define VANESSA_SOCKET_NO_LOOKUP       0x00000001
 #define VANESSA_SOCKET_NO_FROM         0x00000002
 #define VANESSA_SOCKET_NO_FORK         0x00000004
+#define VANESSA_SOCKET_TCP_KEEPALIVE   0x00000008
 
 #define VANESSA_SOCKET_PROTO_MASK      0x0000ff00
 #define __VANESSA_SOCKET_PROTO(_proto)   ((_proto&0xff)<<8)
@@ -184,22 +189,6 @@ int vanessa_socket_client_src_open(const char *src_host,
 				   const vanessa_socket_flag_t flag);
 
 
-/**********************************************************************
- * vanessa_socket_port_portno
- * port number of a service given as a string either
- * the port number or the service name as per /etc/services
- * pre: port name as per /etc/services or port number as a string
- *      flag: Flags. If the VANESSA_SOCKET_NO_LOOKUP bit is set then
- *            no service lookups will be performed. That is the
- *            port given as an argument should be an port number
- * return: port number
- *         -1 on error or if port name cannot be found in /etc/services
- **********************************************************************/
-
-long int vanessa_socket_port_portno(const char *port,
-			       const vanessa_socket_flag_t flag);
-
-
 /**********************************************************************
  * vanessa_socket_str_is_digit
  * Test if a null terminated string is composed entirely of digits (0-9)
@@ -304,10 +293,12 @@ ssize_t vanessa_socket_pipe_fd_write(int fd, const void *buf, size_t count,
  *      buffer_length: size of buffer in bytes
  *      idle_timeout:  timeout in seconds to wait for input
  *                     timeout of 0 = infinite timeout
- *      return_a_read_bytes: Pointer to int where number
+ *      return_a_read_bytes: Pointer to size_t where number
  *                           of bytes read from a will be recorded.
- *      return_b_read_bytes: Pointer to int where number
+ *                           Note that this may wrap
+ *      return_b_read_bytes: Pointer to size_t where number
  *                           of bytes read from b will be recorded.
+ *                           Note that this may wrap
  *      read_func: Function to use for low level reading.
  *                 If NULL, a simple wrapper around read(2) is used
  *      write_func: Function to use for low level writing.
@@ -326,11 +317,11 @@ int vanessa_socket_pipe_func(int rfd_a,
 			     int wfd_a,
 			     int rfd_b,
 			     int wfd_b,
-			     unsigned char *buffer,
+			     char *buffer,
 			     int buffer_length,
 			     int idle_timeout,
-			     int *return_a_read_bytes,
-			     int *return_b_read_bytes,
+			     size_t *return_a_read_bytes,
+			     size_t *return_b_read_bytes,
 			     ssize_t(*read_func) (int fd, void *buf,
 						  size_t count,
 						  void *data),
@@ -414,9 +405,9 @@ int vanessa_socket_pipe_func(int rfd_a,
  *         -1 on error
  **********************************************************************/
 
-int vanessa_socket_pipe_read_write_func(int rfd,
+ssize_t vanessa_socket_pipe_read_write_func(int rfd,
 					int wfd,
-					unsigned char *buffer,
+					char *buffer,
 					int buffer_length,
 					ssize_t(*read_func) (int fd,
 							     void *buf,
@@ -449,7 +440,7 @@ int vanessa_socket_pipe_read_write_func(int rfd,
     rfd, wfd, \
     buffer, buffer_length, \
     vanessa_socket_pipe_fd_read, vanessa_socket_pipe_fd_write, \
-    NULL, NULL \
+    NULL \
   )
 
 
@@ -467,7 +458,7 @@ int vanessa_socket_pipe_read_write_func(int rfd,
  **********************************************************************/
 
 int vanessa_socket_pipe_write_bytes_func(int fd,
-					 const unsigned char *buffer,
+					 const char *buffer,
 					 const ssize_t n,
 					 ssize_t(*write_func) (int fd,
 							       const void
@@ -512,6 +503,26 @@ int vanessa_socket_server_bind(const char *port,
 				     vanessa_socket_flag_t flag);
 
 
+/**********************************************************************
+ * vanessa_socket_server_bindv
+ * Open sockets and bind them to a ports and address
+ * pre: fromv: NULL terminated pointer of interface addresses and ports
+ *             [addr1, port1, addr2, port1, ..., addrN, portN, NULL]
+ *             If you want to bind to "0.0.0.0" specify it litreally, rather
+ *             than as NULL as works with vanessa_socket_server_bind.
+ *             Note: binding to "0.0.0.0" and other addresses may not work
+ *             according to your operating system.
+ *      flag: passed to vanessa_socket_server_bind
+ * return: -1 terminated pointer of bound sockets
+ *         To close the sockets and free, call vanessa_socket_closev();
+ *         NULL on error
+ **********************************************************************/
+
+int *
+vanessa_socket_server_bindv(const const char **fromv,
+                            vanessa_socket_flag_t flag);
+
+
 /**********************************************************************
  * vanessa_socket_server_bind_sockaddr_in
  * Open a socket and bind it to a port and address
@@ -526,30 +537,66 @@ int vanessa_socket_server_bind_sockaddr_in(struct sockaddr_in from,
 					   vanessa_socket_flag_t flag);
 
 
+/**********************************************************************
+ * vanessa_socket_server_bind_sockaddr_inv
+ * Open sockets and bind them to a ports and address
+ * pre: fromv: non-terminated pointer of sockaddr_in to bind to
+ *             [sockaddr1, sockaddr2,..., sockaddrN]
+ *             N.B: This would be NULL terminated, as per
+ *                  vanessa_socket_server_bind_sockaddrv, 
+ *                  but you can't have a NULL struct sockaddr_in
+ *      nfrom: Number of elements in fromv
+ *      flag: ignored
+ * post: Bound socket
+ * return: -1 terminated pointer of bound sockets
+ *         To close the sockets and free, call vanessa_socket_closev();
+ *         NULL on error
+ **********************************************************************/
+
+int *
+vanessa_socket_server_bind_sockaddr_inv(struct sockaddr_in *fromv, 
+					size_t nfrom, 
+					vanessa_socket_flag_t flag);
+
+
+/**********************************************************************
+ * vanessa_socket_closev
+ * Close sockets
+ * pre: fromv: -1 terminated pointer of sockets
+ *             [sockaddr1, sockaddr2,..., sockaddrN, NULL]
+ *      flag: ignored
+ * post: Sockets are closed
+ *       sockv is freed
+ * return: The first non-zero return status returned by an call to close()
+ *         0 if all close() calls were successful
+ **********************************************************************/
+
+int
+vanessa_socket_closev(int *sockv);
+
+
 /**********************************************************************
  * vanessa_socket_server_accept
  * Accept connections on a bound socket.
  * vanessa_socket_server_bind or vanessa_socket_server_bind_sockaddr_in
  * may be used to open the bound socket.
  * When one is received fork
- * In the Child: close the listening file descriptor
+ * In the Child: close listen_socket and
  *               return the file descriptor that is 
  *               a socket connection to the client
  * In the Server: close the socket to the client and loop
- * pre: port: port to listen to, an ASCII representation of a number
- *            or an entry from /etc/services
- *      interface_address: If NULL bind to all interfaces, else
- *                         bind to interface(es) with this address.
+ * pre: listen_socket: socket to listen for connection on
  *      maximum_connections: maximum number of active connections
  *                           to handle. If 0 then an number of connections 
  *                           is unlimited.
- *      return_from: pointer to a struct_in addr where the 
+ *                           Not used if flag is VANESSA_SOCKET_NO_FORK
+ *      return_from: pointer to a struct sockaddr where the 
  *                   connecting client's IP address will
  *                   be placed. Ignored if NULL
- *      return_to: pointer to a in addr where the IP address the 
+ *      return_to: pointer to a struct sockaddr where the IP address the 
  *                 server accepted the connection on will be placed.
  *                 Ignored if NULL
- *      flag: If VANESSA_SOCKET_NO_FORK then the process does not for
+ *      flag: If VANESSA_SOCKET_NO_FORK then the process does not fork
  *            when a connection is recieved.
  * post: Client sockets are returned in child processes
  *       In the parent process the function doesn't exit, other 
@@ -561,8 +608,49 @@ int vanessa_socket_server_bind_sockaddr_in(struct sockaddr_in from,
 
 int vanessa_socket_server_accept(int listen_socket,
 				      const unsigned int maximum_connections,
-				      struct sockaddr_in *return_from, 
-				      struct sockaddr_in *return_to,
+				      struct sockaddr *return_from, 
+				      struct sockaddr *return_to,
+				      vanessa_socket_flag_t flag);
+
+
+/**********************************************************************
+ * vanessa_socket_server_acceptv
+ * Accept connections on a bound socket.
+ * vanessa_socket_server_bind or vanessa_socket_server_bind_sockaddr_in
+ * may be used to open the bound socket.
+ * When one is received fork
+ * In the Child: close each socket in listen_socketv and
+ *               return the file descriptor that is 
+ *               a socket connection to the client
+ * In the Server: close the socket to the client and loop
+ * pre: port: port to listen to, an ASCII representation of a number
+ *            or an entry from /etc/services
+ *      listen_socketc: -1 terminated pointer to sockets to listen on
+ *      maximum_connections: maximum number of active connections
+ *                           to handle. If 0 then an number of connections 
+ *                           is unlimited.
+ *                           Not used if flag is VANESSA_SOCKET_NO_FORK
+ *      return_from: pointer to a struct sockaddr where the 
+ *                   connecting client's IP address will
+ *                   be placed. Ignored if NULL
+ *      return_to: pointer to a struct sockaddr where the IP address the 
+ *                 server accepted the connection on will be placed.
+ *                 Ignored if NULL
+ *      flag: If VANESSA_SOCKET_NO_FORK then the process does not fork
+ *            when a connection is recieved.
+ * post: Client sockets are returned in child processes
+ *       In the parent process the function doesn't exit, other 
+ *       than on error.
+ *       if return_from is non-null, it is seeded with cleints address
+ * return: client socket, if connection is accepted.
+ *         -1 on error
+ **********************************************************************/
+
+int 
+vanessa_socket_server_acceptv(int *listen_socketv,
+				      const unsigned int maximum_connections,
+				      struct sockaddr *return_from, 
+				      struct sockaddr *return_to,
 				      vanessa_socket_flag_t flag);
 
 
@@ -600,22 +688,27 @@ int vanessa_socket_server_accept(int listen_socket,
 int vanessa_socket_server_connect(const char *port,
 				  const char *interface_address,
 				  const unsigned int maximum_connections,
-				  struct sockaddr_in *return_from,
-				  struct sockaddr_in *return_to,
+				  struct sockaddr *return_from,
+				  struct sockaddr *return_to,
 				  vanessa_socket_flag_t flag);
 
 
 /**********************************************************************
- * vanessa_socket_server_connect_sockaddr_in
+ * vanessa_socket_server_connectv
  * Listen on a tcp port for incoming client connections 
  * When one is received fork
  * In the Child: close the listening file descriptor
  *               return the file descriptor that is 
  *               a socket connection to the client
  * In the Server: close the socket to the client and loop
- * pre: from: sockaddr_in to bind to
+ * pre: fromv: NULL terminated pointer of interface addresses and ports
+ *             [addr1, port1, addr2, port1, ..., addrN, portN, NULL]
+ *             If you want to bind to "0.0.0.0" specify it litreally, rather
+ *             than as NULL as works with vanessa_socket_server_connect.
+ *             Note: binding to "0.0.0.0" and other addresses may not work
+ *             according to your operating system.
  *      maximum_connections: maximum number of active connections to
- *                           handle. If 0 then an number of connections
+ *                           handle. If 0 then an number of connections 
  *                           is unlimited.
  *      return_from: pointer to a struct_in addr where the 
  *                   connecting client's IP address will
@@ -623,7 +716,8 @@ int vanessa_socket_server_connect(const char *port,
  *      return_to: pointer to a in addr where the IP address the 
  *                 server accepted the connection on will be placed.
  *                 Ignored if NULL
- *      flag: ignored
+ *      flag: If VANESSA_SOCKET_NO_LOOKUP then no host and port lookups
+ *            will be performed
  * post: Client sockets are returned in child processes
  *       In the parent process the function doesn't exit, other 
  *       than on error.
@@ -632,14 +726,12 @@ int vanessa_socket_server_connect(const char *port,
  *         -1 on error
  **********************************************************************/
 
-int vanessa_socket_server_connect_sockaddr_in(struct sockaddr_in from,
-					      const unsigned int
-					      maximum_connections,
-					      struct sockaddr_in
-					      *return_from,
-					      struct sockaddr_in
-					      *return_to,
-					      vanessa_socket_flag_t flag);
+int 
+vanessa_socket_server_connectv(const const char **from,
+			       const unsigned int maximum_connections,
+			       struct sockaddr *return_from,
+			       struct sockaddr *return_to,
+			       vanessa_socket_flag_t flag);
 
 
 /**********************************************************************
@@ -655,55 +747,6 @@ int vanessa_socket_server_connect_sockaddr_in(struct sockaddr_in from,
 void vanessa_socket_server_reaper(void);
 
 
-/**********************************************************************
- * vanessa_socket_host_in_addr
- * A host is given as a string either as a host name or IP address
- * as a dotted quad. The host is used to seed an in_addr structure
- * with a binary representation of the IP address of the host
- * in network byte order.
- * pre: host: hostname or IP address
- *            If NULL then INADDR_ANY will be converted to network
- *            byte order and used as the address.
- *      in: pointer to an in_addr structure.
- *      flag: Flags. If the VANESSA_SOCKET_NO_LOOKUP bit is set then
- *            no hostname lookups will be performed. That is the
- *            host given as an argument should be an IP address
- * post: none
- * return: 0 on success
- *         -1 on error
- **********************************************************************/
-
-int vanessa_socket_host_in_addr(const char *host,
-				struct in_addr *in,
-				const vanessa_socket_flag_t flag);
-
-
-/**********************************************************************
- * vanessa_socket_host_port_sockaddr_in
- * A host is given as a string either as a host name or IP address
- * as a dotted quad. A port number of a service is given as either a the
- * port number of the service name as per /etc/services. This is used
- * to seed a sockaddr_in structure.
- * pre: host: hostname or IP address
- *            If NULL then INADDR_ANY will be converted to network
- *            byte order and used as the address.
- *      port: port name as per /etc/services or port number as a string
- *      addr: pointer to an sockaddr_in structure.
- *      flag: Flags. If the VANESSA_SOCKET_NO_LOOKUP bit is set then
- *            no lookups will be performed. That is the
- *            host given as an argument should be an IP address and
- *            the port should be a port number
- * post: none
- * return: 0 on success
- *         -1 on error
- **********************************************************************/
-
-int vanessa_socket_host_port_sockaddr_in(const char *host,
-					 const char *port,
-					 struct sockaddr_in *addr,
-					 const vanessa_socket_flag_t flag);
-
-
 /**********************************************************************
  * Logging functionality
  *
